// testview.cpp : implementation of the CTestView class
//

#include "stdafx.h"
#include "test.h"

#include "testdoc.h"
#include "testview.h"
#include "amber.hpp"
#include "lightdlg.h"
#include "clipdlg.h"
#include "texture.hpp"
#include "viodlg.h"
#include "fogdlg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

extern universeClass *univ;
extern CLightDlg *lightDlg;
extern CString launchFile;
extern CFogDlg *fogDlg;

CVioDlg *vioDlg=NULL;

/////////////////////////////////////////////////////////////////////////////
// CTestView

IMPLEMENT_DYNCREATE(CTestView, CView)

BEGIN_MESSAGE_MAP(CTestView, CView)
	//{{AFX_MSG_MAP(CTestView)
	ON_WM_SIZE()
	ON_COMMAND(ID_LIGHT_CONTROL, OnLightControl)
	ON_UPDATE_COMMAND_UI(ID_LIGHT_CONTROL, OnUpdateLightControl)
	ON_COMMAND(ID_RENDER_BBOX, OnRenderBbox)
	ON_UPDATE_COMMAND_UI(ID_RENDER_BBOX, OnUpdateRenderBbox)
	ON_COMMAND(ID_RENDER_LINES, OnRenderLines)
	ON_UPDATE_COMMAND_UI(ID_RENDER_LINES, OnUpdateRenderLines)
	ON_COMMAND(ID_RENDER_MATERIAL, OnRenderMaterial)
	ON_UPDATE_COMMAND_UI(ID_RENDER_MATERIAL, OnUpdateRenderMaterial)
	ON_COMMAND(ID_RENDER_NORMALS, OnRenderNormals)
	ON_UPDATE_COMMAND_UI(ID_RENDER_NORMALS, OnUpdateRenderNormals)
	ON_COMMAND(ID_RENDER_PERF, OnRenderPerf)
	ON_COMMAND(ID_RENDER_POINTS, OnRenderPoints)
	ON_UPDATE_COMMAND_UI(ID_RENDER_POINTS, OnUpdateRenderPoints)
	ON_COMMAND(ID_RENDER_POLYS, OnRenderPolys)
	ON_UPDATE_COMMAND_UI(ID_RENDER_POLYS, OnUpdateRenderPolys)
	ON_COMMAND(ID_RENDER_TRIS, OnRenderTris)
	ON_UPDATE_COMMAND_UI(ID_RENDER_TRIS, OnUpdateRenderTris)
	ON_COMMAND(ID_VP_DECLIN, OnVpDeclin)
	ON_COMMAND(ID_VP_DECROT, OnVpDecrot)
	ON_COMMAND(ID_VP_INCLIN, OnVpInclin)
	ON_COMMAND(ID_VP_INCROT, OnVpIncrot)
	ON_COMMAND(ID_RENDER_SHADING, OnRenderShading)
	ON_UPDATE_COMMAND_UI(ID_RENDER_SHADING, OnUpdateRenderShading)
	ON_COMMAND(ID_UNIV_BKGCOLOR, OnUnivBkgcolor)
	ON_COMMAND(ID_VP_CLIPPING, OnVpClipping)
	ON_COMMAND(ID_VP_MOUSE, OnVpMouse)
	ON_UPDATE_COMMAND_UI(ID_VP_MOUSE, OnUpdateVpMouse)
	ON_COMMAND(ID_VP_PLATTER, OnVpPlatter)
	ON_UPDATE_COMMAND_UI(ID_VP_PLATTER, OnUpdateVpPlatter)
	ON_WM_KEYDOWN()
	ON_COMMAND(ID_VIO_RUNNING, OnVioRunning)
	ON_UPDATE_COMMAND_UI(ID_VIO_RUNNING, OnUpdateVioRunning)
	ON_COMMAND(ID_FOG, OnFog)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTestView construction/destruction

CTestView::CTestView()
{
	// TODO: add construction code here
	ch = NULL;
	mouse = NULL;
	platter = NULL;
	useMouse=1;
}

CTestView::~CTestView()
{
	delete mouse;
	delete platter;
}

/////////////////////////////////////////////////////////////////////////////
// CTestView drawing

void CTestView::OnDraw(CDC* pDC)
{
	CTestDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CTestView printing

BOOL CTestView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CTestView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CTestView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CTestView diagnostics

#ifdef _DEBUG
void CTestView::AssertValid() const
{
	CView::AssertValid();
}

void CTestView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CTestDoc* CTestView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTestDoc)));
	return (CTestDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTestView message handlers

void CTestView::OnInitialUpdate() 
{
	// TODO: Add your specialized code here and/or call the base class
	HWND hwnd;
	CView::OnInitialUpdate();
	geometryClass *g;

	if (mouse == NULL) {
		mouse   = new mouseClass(m_hWnd);
	}

	if (platter == NULL) {
		platter   = new platterClass(m_hWnd);
	}

	if (ch == NULL) {
		hwnd = GetSafeHwnd();
		ch = new channelClass((void *)hwnd);
		mouse->attachChannel(ch);
	}

	if (launchFile.GetLength() > 0) {
		g = new geometryClass(launchFile.GetBuffer(launchFile.GetLength()));
		launchFile = "";
	}

}

BOOL CTestView::PreCreateWindow(CREATESTRUCT& cs) 
{
	// TODO: Add your specialized code here and/or call the base class
	
	cs.style = cs.style | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

	return CView::PreCreateWindow(cs);
}

void CTestView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	if (!IsWindowVisible() || ch==NULL) return;
	ch->resetPerspective();
	
}

void CTestView::OnLightControl() 
{
	// TODO: Add your command handler code here
	if (!lightDlg->IsWindowVisible()) 
		lightDlg->ShowWindow(SW_SHOW);
	else
		lightDlg->ShowWindow(SW_HIDE);
	
}

void CTestView::OnUpdateLightControl(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(lightDlg->IsWindowVisible());	
}

void CTestView::OnRenderBbox() 
{
	// TODO: Add your command handler code here
	if (ch->getState() & RENDER_BOUNDINGBOX)
		ch->disableState(RENDER_BOUNDINGBOX);
	else
		ch->enableState(RENDER_BOUNDINGBOX);
	
	
}

void CTestView::OnUpdateRenderBbox(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(ch->getState() & RENDER_BOUNDINGBOX);	
	
}

void CTestView::OnRenderLines() 
{
	// TODO: Add your command handler code here
	if (ch->getPolyFill() == _NOFILL)
		ch->setPolyFill(_FILL);
	else
		ch->setPolyFill(_NOFILL);
	
	
}

void CTestView::OnUpdateRenderLines(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(ch->getPolyFill() == _NOFILL);	
	
}

void CTestView::OnRenderMaterial() 
{
	// TODO: Add your command handler code here
	if (ch->getState() & RENDER_MATERIAL)
		ch->disableState(RENDER_MATERIAL);
	else
		ch->enableState(RENDER_MATERIAL);
	
}

void CTestView::OnUpdateRenderMaterial(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(ch->getState() & RENDER_MATERIAL);	
	
}

void CTestView::OnRenderNormals() 
{
	// TODO: Add your command handler code here
	if (ch->getState() & RENDER_NORMAL)
		ch->disableState(RENDER_NORMAL);
	else
		ch->enableState(RENDER_NORMAL);
	
	
}

void CTestView::OnUpdateRenderNormals(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(ch->getState() & RENDER_NORMAL);	
	
}

void CTestView::OnRenderPerf() 
{
	// TODO: Add your command handler code here
	ch->viewPerformance();	
	
}

void CTestView::OnRenderPoints() 
{
	// TODO: Add your command handler code here
	ch->setPolyRenderAs(_POINTS);
	
}

void CTestView::OnUpdateRenderPoints(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck((ch->getPolyRenderAs() == _POINTS));	
	
}

void CTestView::OnRenderPolys() 
{
	// TODO: Add your command handler code here
	ch->setPolyRenderAs(_POLYS);
	
}

void CTestView::OnUpdateRenderPolys(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(ch->getPolyRenderAs() == _POLYS);	
	
}

void CTestView::OnRenderTris() 
{
	// TODO: Add your command handler code here
	ch->setPolyRenderAs(_TRIS);
	
}

void CTestView::OnUpdateRenderTris(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(ch->getPolyRenderAs() == _TRIS);	
	
}

void CTestView::OnVpDeclin() 
{
	// TODO: Add your command handler code here
	if (useMouse)
		mouse->setLinearScaling(0.9*mouse->getLinearScaling());
	else
		platter->setLinearScaling(0.9*mouse->getLinearScaling());
	
}

void CTestView::OnVpDecrot() 
{
	// TODO: Add your command handler code here
	if (useMouse)
		mouse->setRotationalScaling(0.9*mouse->getRotationalScaling());
	else
		platter->setRotationalScaling(0.9*mouse->getRotationalScaling());
}

void CTestView::OnVpInclin() 
{
	// TODO: Add your command handler code here
	if (useMouse)
		mouse->setLinearScaling(1.2*mouse->getLinearScaling());
	else
		platter->setLinearScaling(1.2*mouse->getLinearScaling());
	
}

void CTestView::OnVpIncrot() 
{
	// TODO: Add your command handler code here
	if (useMouse)
		mouse->setRotationalScaling(1.2*mouse->getRotationalScaling());
	else
		platter->setRotationalScaling(1.2*mouse->getRotationalScaling());

}

void CTestView::OnRenderShading() 
{
	// TODO: Add your command handler code here
	if (ch->getPolyShading() == _SHADED)
		ch->setPolyShading(_FLAT);
	else
		ch->setPolyShading(_SHADED);
	
	
}

void CTestView::OnUpdateRenderShading(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck((ch->getPolyShading() == _SHADED));	
	
}

void CTestView::OnUnivBkgcolor() 
{
	// TODO: Add your command handler code here
	F4 color;
	COLORREF cr;
	CColorDialog dlg;

	ch->getBackgroundColor(color);
	
	dlg.m_cc.Flags |= CC_RGBINIT;
	dlg.m_cc.rgbResult = RGB( (int)(color[0]*255.0), 
	                          (int)(color[1]*255.0), 
	                          (int)(color[2]*255.0));
	//dlg.SetCurrentColor(cr);

	if (!dlg.DoModal()) return;

	cr = dlg.GetColor();
	V4_init(color);
	color[0] = (float)GetRValue(dlg.m_cc.rgbResult)/255.0;
	color[1] = (float)GetGValue(dlg.m_cc.rgbResult)/255.0;
	color[2] = (float)GetBValue(dlg.m_cc.rgbResult)/255.0;
	ch->setBackgroundColor(color);
	
}

void CTestView::OnVpClipping() 
{
	// TODO: Add your command handler code here
	CClipDlg dlg;
	
	dlg.DoModal();	
	
}

void CTestView::OnVpMouse() 
{
	// TODO: Add your command handler code here
	if (!useMouse) {
		platter->removeChannel(ch);
		useMouse=TRUE;
		mouse->attachChannel(ch);
	}	
}

void CTestView::OnUpdateVpMouse(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(useMouse);	
}

void CTestView::OnVpPlatter() 
{
	// TODO: Add your command handler code here
	if (useMouse) {
		mouse->removeChannel(ch);
		useMouse=FALSE;
		platter->attachChannel(ch);
	}	
	useMouse=0;
	
}

void CTestView::OnUpdateVpPlatter(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(!useMouse);	
	
}

void CTestView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default

	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}


void CTestView::OnVioRunning() 
{
	// TODO: Add your command handler code here
	if (vioDlg == NULL) {
		mouse->removeChannel(ch);
		vioDlg = new CVioDlg();
	}
	else {
		delete vioDlg->mouse;
		delete vioDlg->ch;
		delete vioDlg->vio;
		delete vioDlg;
		vioDlg = NULL;
		mouse->attachChannel(ch);
		amber->setRenderChannel(ch);
	}
}

void CTestView::OnUpdateVioRunning(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(vioDlg != NULL);
}

void CTestView::OnFog() 
{
	// TODO: Add your command handler code here
	if (!fogDlg->IsWindowVisible()) 
		fogDlg->ShowWindow(SW_SHOW);
	else
		fogDlg->ShowWindow(SW_HIDE);
	
}
